CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
109 lines
2.6 KiB
C++
109 lines
2.6 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#include <iostream>
|
|
|
|
// hip header file
|
|
#include "hip/hip_runtime.h"
|
|
#include "hip_helper.h"
|
|
|
|
#define LENGTH 4
|
|
|
|
#define SIZE (LENGTH * LENGTH)
|
|
|
|
#define THREADS_PER_BLOCK 1
|
|
#define BLOCKS_PER_GRID LENGTH
|
|
|
|
// CPU function - basically scan each row and save the output in array
|
|
void matrixRowSum(int* input, int* output, int width) {
|
|
for (int i = 0; i < width; i++) {
|
|
for (int j = 0; j < width; j++) {
|
|
output[i] += input[i * width + j];
|
|
}
|
|
}
|
|
}
|
|
|
|
// Device (kernel) function
|
|
__global__ void gpuMatrixRowSum(int* input, int* output, int width) {
|
|
int index = blockDim.x * blockIdx.x + threadIdx.x;
|
|
#pragma unroll
|
|
for (int i = 0; i < width; i++) {
|
|
output[index] += input[index * width + i];
|
|
}
|
|
}
|
|
|
|
int main() {
|
|
int* Matrix;
|
|
int* sumMatrix;
|
|
int* cpuSumMatrix;
|
|
|
|
int* gpuMatrix;
|
|
int* gpuSumMatrix;
|
|
|
|
hipDeviceProp_t devProp;
|
|
checkHipErrors(hipGetDeviceProperties(&devProp, 0));
|
|
|
|
std::cout << "Device name " << devProp.name << std::endl;
|
|
|
|
Matrix = (int*)malloc(sizeof(int) * SIZE);
|
|
sumMatrix = (int*)malloc(sizeof(int) * LENGTH);
|
|
cpuSumMatrix = (int*)malloc(sizeof(int) * LENGTH);
|
|
|
|
for (int i = 0; i < SIZE; i++) {
|
|
Matrix[i] = i * 2;
|
|
}
|
|
|
|
for (int i = 0; i < LENGTH; i++) {
|
|
cpuSumMatrix[i] = 0;
|
|
}
|
|
|
|
// Allocated Device Memory
|
|
checkHipErrors(hipMalloc((void**)&gpuMatrix, SIZE * sizeof(int)));
|
|
checkHipErrors(hipMalloc((void**)&gpuSumMatrix, LENGTH * sizeof(int)));
|
|
|
|
// Memory Copy to Device
|
|
checkHipErrors(hipMemcpy(gpuMatrix, Matrix, SIZE * sizeof(int), hipMemcpyHostToDevice));
|
|
checkHipErrors(
|
|
hipMemcpy(gpuSumMatrix, cpuSumMatrix, LENGTH * sizeof(float), hipMemcpyHostToDevice));
|
|
|
|
// Launch device kernels
|
|
hipLaunchKernelGGL(gpuMatrixRowSum, dim3(BLOCKS_PER_GRID), dim3(THREADS_PER_BLOCK), 0, 0,
|
|
gpuMatrix, gpuSumMatrix, LENGTH);
|
|
|
|
// Memory copy back to device
|
|
checkHipErrors(hipMemcpy(sumMatrix, gpuSumMatrix, LENGTH * sizeof(int), hipMemcpyDeviceToHost));
|
|
|
|
// Cpu implementation
|
|
matrixRowSum(Matrix, cpuSumMatrix, LENGTH);
|
|
|
|
|
|
// verify the results
|
|
int errors = 0;
|
|
for (int i = 0; i < LENGTH; i++) {
|
|
if (sumMatrix[i] != cpuSumMatrix[i]) {
|
|
printf("%d - cpu: %d gpu: %d\n", i, sumMatrix[i], cpuSumMatrix[i]);
|
|
errors++;
|
|
}
|
|
}
|
|
|
|
if (errors == 0) {
|
|
printf("PASSED\n");
|
|
} else {
|
|
printf("FAILED with %d errors\n", errors);
|
|
}
|
|
|
|
// GPU Free
|
|
checkHipErrors(hipFree(gpuMatrix));
|
|
checkHipErrors(hipFree(gpuSumMatrix));
|
|
|
|
// CPU Free
|
|
free(Matrix);
|
|
free(sumMatrix);
|
|
free(cpuSumMatrix);
|
|
|
|
return errors;
|
|
}
|